// Mental Trousers 10 Aug 2000
//---------------------------------------------------------------------------
// clientMessage.cs
//
// This script provides exceptionally fast parsing of the incoming client
// messages.
//
// Normally, scripts would attach to the eventClientMessage and use
// match::string, match::paramString, string::findSubStr etc to look for
// a matching piece of text. Each one of those functions start at the first
// character of the incoming message and step through it one character at a
// time. If there are 50 match::string's looking for 50 different pieces of
// text, then the incoming message is parsed 50 times.
//
// This script does a single character by character pass through the incoming
// message, triggering functions provided by other scripts as it finds a
// match.
//
// msg::onMatch (%text, %func, %wildcard)
//       %text is the text to be matched.
//       %func is the function to be called OR string to be evaluated when
//         a match for text is found.
//       %wildcard is the wildcard character to use. If this is "" then
//         the * is used as the wildcard.
//         Also, if a string is provided rather than a function name, the
//         client and message can be passed by using %client and %msg as
//         function parameters in the string.
//
// msg::remove (%text, %func)
//         this is the function that scripts can use to remove or detach
//         from the parser. The %text and %func must be an exact match for
//         the parameters provided to msg::onMatch or this will fail.
//         returns true if successful, false on fail.
//---------------------------------------------------------------------------

function msg::onMatch (%text, %func, %wildcard)
{
	if (%text=="" || %func=="")
		return;

	if (%wildcard=="")
		%wildcard="*";

	//I have no idea why, but heaps of stuff gets initialised twice so I've
	//had to put in a safety net. Here it is.
	if ($safety::[%text, %func, %wildcard])
		return;

	$safety::[%text, %func, %wildcard]=true;
	if (string::findSubStr (%func, %wildcard)!=-1)
		msg::onSafeMatch (%text, %func, %wildcard);
	else
		msg::onMatch_ (%text, %func, %wildcard);
	return;
}

function msg::onSafeMatch (%text, %func, %wildcard)
{
	%string=%func;
	if (string::findSubStr (%func, ";")==-1)
		%string=%string@"(%client, %msg);";

	%string="msg::safeCall(\""@%string@"\", %client, %msg);";
	msg::onMatch_ (%text, %string, %wildcard);
	return;
}

function msg::onMatch_ (%text, %func, %wildcard)
{
	if (%text=="" || %func=="")
		return;

	if (%wildcard=="")
		%wildcard="*";

	%string=%func;
	if (string::findSubStr (%func, ";")==-1)
		%string=%string@"(%client, %msg);";

	%chars="abcdefghijklmnopqrstuvwxyz!@#$%^&()_+-=,./<>?|;:[]{} `~\\*";
	%len=String::len(%text);
	%node=0;
	for (%i=0; %i<string::len (%text); %i++)
		{
		%thisLetter=string::getSubStr (%text, %i, 1);

		if ($msg::[%node, %thisLetter]=="" || %thisLetter=="*")
			{
			%newNode=msg::newNode();
			$msg::[%node, %thisLetter]=%newNode;
			if (%thisLetter=="*")
				{
				for (%j=0; %j<string::len(%chars); %j++)
					{
					%temp=string::getSubStr (%chars, %j, 1);
					if (%i<%len-1)
						{
						if (%node==0)
							$msg::[%node, %temp, "string"]=$msg::[%node, %temp, "string"]@"msg::eval ("@%newNode@", msg::letter());";
						else
							$msg::[%node, %temp, "string"]=$msg::[%node, %temp, "string"]@"msg::eval ("@%newNode@", msg::letter(), "@%node@");";
						if ($msg::[%node, %temp]=="")
							$msg::[%node, %temp]=%newNode;
						}
					else
						{
						$msg::[%node, %temp, "string"]=$msg::[%node, %temp, "string"]@%string;
						$msg::final::[%text]=%node;
						}
					}
				}
			else
				{
				if (%i<%len-1)
					$msg::[%node, %thisLetter, "string"]=$msg::[%node, %thisLetter, "string"]@"msg::eval ("@%newNode@", msg::letter());";
				else
					{
					$msg::[%node, %thisLetter, "string"]=$msg::[%node, %thisLetter, "string"]@%string;
					$msg::final::[%text]=%node;
					}
				}
			}
		%oldNode=%node;
		%node=$msg::[%node, %thisLetter];
		}
	return;
}

function msg::remove (%text, %func)
{
	%string=%func;
	if (string::findSubStr (%string, ";")==-1)
		%string=%string@"(%client, %msg);";

	%letter=string::getSubStr (%text, string::len(%text)-1, 1);
	%node=$msg::final::[%text];
	%nodeStr=$msg::[%node, %letter, "string"];
	$msg::[%node, %letter, "string"]=string::replace (%nodeStr, %string, "");
	return (%nodeStr!=$msg::[%node, %letter, "string"]);
}

function msg::safeCall (%string, %client, %msg)
{
	schedule::add ("msg::safeCall_(\""@%string@"\", \""@%client@"\", \""@%msg@"\");", 0.1);
	return;
}

function msg::safeCall_ (%string, %client, %msg)
{
	eval (%string);
	return;
}

function msg::incoming (%client, %msg)
{
	event::trigger (eventClientMessagePreParse);
	%len=string::len(%msg);
	$eval_="msg::eval(0, msg::letter());";
	for (%i=0;%i<%len;%i++)
		{
		$msg::letter=string::getSubStr (%msg, %i, 1);
		%eval_=$eval_;
		$eval_="msg::eval(0, msg::letter());";
		eval (%eval_);
		}
	%eval_=$eval_;
	eval (%eval_);
	%return=event::trigger (eventClientMessagePostParse);
	if (Event::Returned(%return, "mute"))
		return "mute";
	return true;
}

function msg::eval (%node, %letter, %repeat)
{
	$eval_=$eval_@$msg::[%node, %letter, "string"];
	if (%repeat)
		$eval_=$eval_@"msg::eval ("@%node@", msg::letter(), "@%repeat@");";
}

function msg::letter()
{
	return $msg::letter;
}

function msg::newNode()
{
	return $msg::nextNode++;
}

event::attach (eventClientMessage, msg::incoming);
